/*  */

#include "platform.h"
#include "xaxidma.h"
#include "xparameters.h"
#include "xdebug.h"

#include "xgpiops.h" // for PS GPIO(MIO)
#include "xgpio.h" // for SW, LED

#define LED_PIN 7
#define PB1_PIN 50
#define PB2_PIN 51

#define BLKSIZE		1024
#define NUMBLK		8


/* ydvz݂╡Xbhǂݏϐvolatileɂ */
typedef u32 dataunit;

volatile XAxiDma AxiDma;
volatile XGpioPs gpiops;
volatile unsigned int *GPIO_wc =XPAR_AXI_GPIO_0_BASEADDR+XGPIO_DATA_OFFSET;
volatile unsigned int *GPIO_rc =XPAR_AXI_GPIO_0_BASEADDR+XGPIO_DATA2_OFFSET;

/* ydvzcl`]̌Ɛ̓́uL̗ǂvꏊɔzu */
volatile dataunit txbuf[BLKSIZE*NUMBLK] __attribute__((aligned(64)));
volatile dataunit rxbuf[BLKSIZE*NUMBLK] __attribute__((aligned(64)));

volatile int pb1,pb2,led;

int XAxiDma_SimplePollExample(u16 DeviceId);

int main(){
  XGpioPs_Config* gpiops_config_ptr;
  int i;
  int r;

  init_platform();

  print("src timestamp: ");
  print(__TIMESTAMP__);
  print("\r\n");

  print("---- Start ---- \r\n");
  
  gpiops_config_ptr=XGpioPs_LookupConfig(XPAR_XGPIOPS_0_DEVICE_ID);
  r=XGpioPs_CfgInitialize
    (&gpiops,gpiops_config_ptr,gpiops_config_ptr->BaseAddr);
  if(r!=XST_SUCCESS) {
    printf( "GPIOps Init Err\r\n");
    exit(-1);
  }
  XGpioPs_SetDirectionPin   (&gpiops,LED_PIN,1);
  XGpioPs_SetDirectionPin   (&gpiops,PB1_PIN,0);
  XGpioPs_SetDirectionPin   (&gpiops,PB2_PIN,0);
  XGpioPs_SetOutputEnablePin(&gpiops,LED_PIN,1);
  XGpioPs_SetOutputEnablePin(&gpiops,PB1_PIN,0);
  XGpioPs_SetOutputEnablePin(&gpiops,PB2_PIN,0);

  /* Run the poll example for simple transfer */
  r = XAxiDma_SimplePollExample(XPAR_AXIDMA_0_DEVICE_ID);
  if (r == XST_SUCCESS) {
    print("DMA test: Passed\r\n");
  } else {
    print("DMA test: Failed\r\n");
  }
  
  while (1) {
    for (i=0;i<0x800000;i++)/* Delay */
      XGpioPs_WritePin(&gpiops,LED_PIN,((i&0x100000)?1:0)); /* Flash */
    pb1=XGpioPs_ReadPin(&gpiops,PB1_PIN);
    pb2=XGpioPs_ReadPin(&gpiops,PB2_PIN);
    if (pb1||pb2) break;
  }

  print("--- End --- \r\n");

  cleanup_platform();

  return XST_SUCCESS;
}


int XAxiDma_SimplePollExample(u16 DeviceId){
  XAxiDma_Config *CfgPtr;
  int r;
  int i,j,t;
  int dmatxbusy,dmarxbusy;
  int ntx,nrx;
  
  /* Initialize the XAxiDma device */
  CfgPtr = XAxiDma_LookupConfig(DeviceId);
  if (!CfgPtr) {
    printf("No config found for %d\r\n", DeviceId);
    return XST_FAILURE;
  }
  r = XAxiDma_CfgInitialize(&AxiDma, CfgPtr);
  if (r != XST_SUCCESS) {
    printf("Initialization failed %d\r\n", r);
    return XST_FAILURE;
  }
  if(XAxiDma_HasSg(&AxiDma)){
    printf("Device configured as SG mode \r\n");
    return XST_FAILURE;
  }
  XAxiDma_Reset(&AxiDma);
  while (!XAxiDma_ResetIsDone(&AxiDma)){
    printf("Waiting for DMA reset done\r\n");
  }

  /* Disable interrupts, we use polling mode */
  XAxiDma_IntrDisable
    (&AxiDma, XAXIDMA_IRQ_ALL_MASK,XAXIDMA_DEVICE_TO_DMA);
  XAxiDma_IntrDisable
    (&AxiDma, XAXIDMA_IRQ_ALL_MASK,XAXIDMA_DMA_TO_DEVICE);

  for(i = 0; i < NUMBLK*BLKSIZE; i ++) {
    txbuf[i]=0xaa0000aa|(i<<8);
    rxbuf[i]=0x12345678;
  }

  /* set test value for tx */
  for (i=0;i<NUMBLK;i++) {
    for(j=0;j<BLKSIZE;j++) {
      txbuf[i*BLKSIZE+j]=j|0x80;
    }
  }
  /* ydvzcl`őoOɃLbV|o */
  Xil_DCacheFlushRange((u32)txbuf, NUMBLK*BLKSIZE*sizeof(dataunit));

  if ((*GPIO_wc)!=0||(*GPIO_rc)!=0) {
    printf("FIFO is not empty at begining\r\n");
    printf("Reset the system\r\n");
    return XST_FAILURE;    
  }

  ntx=0;nrx=0;
  t=0;
  while (ntx<NUMBLK || nrx<NUMBLK || dmatxbusy || dmarxbusy) {

    /* ydvz\tgEFAIdĺu]rW[mFv */
    /* @@@@urW[mF]vƂPڂs */
    dmatxbusy=(ntx>0)?XAxiDma_Busy(&AxiDma,XAXIDMA_DMA_TO_DEVICE):0;
    dmarxbusy=(nrx>0)?XAxiDma_Busy(&AxiDma,XAXIDMA_DEVICE_TO_DMA):0;
    printf("%04xA tx/rx block=%x/%x DMA tx/rx busy=%d/%d FIFO wr/rd cnt=%03x/%03x\r\n",
	   t,ntx,nrx,dmatxbusy,dmarxbusy,*GPIO_wc,*GPIO_rc);

    pb1=XGpioPs_ReadPin(&gpiops,PB1_PIN);
    pb2=XGpioPs_ReadPin(&gpiops,PB2_PIN);

    if (pb1 && pb2) break; /* forced quit */

    if (pb1) {
      if (ntx>=NUMBLK) {
	printf("all blocks has been sent\r\n");
      } else if (ntx>0&&dmatxbusy) {
	/* !!CONFUZING!! first busy must be ignored!! */
	printf("tx DMA busy\r\n");
      } else {
	printf("send %dth block\r\n",ntx);
	r=XAxiDma_SimpleTransfer
	  (&AxiDma,(u32)(txbuf+ntx*BLKSIZE),
	   BLKSIZE*sizeof(dataunit),XAXIDMA_DMA_TO_DEVICE);
	if (r==XST_SUCCESS) {
	  ntx++;
	} else {
	  printf("send failure %d\r\n");
	}
      }
    }

    if (pb2) {
      if (nrx>=NUMBLK) {
	printf("all blocks has been received\r\n");
      } else if (nrx>0&&dmarxbusy) {
	/* !!CONFUZING!! first busy must be ignored!! */
	printf("rx DMA busy\r\n");
      } else {
	printf("recv %dth block\r\n",nrx);
	r=XAxiDma_SimpleTransfer
	  (&AxiDma,(u32)(rxbuf+nrx*BLKSIZE),
	   BLKSIZE*sizeof(dataunit),XAXIDMA_DEVICE_TO_DMA);
	if (r==XST_SUCCESS) {
	  nrx++;
	} else {
	  printf("send failure %d\r\n");
	}
      }
    }

    /* ydvz\tgEFAIdĺu]rW[mFv */
    /* @@@@urW[mF]vƂPڂs */
    dmatxbusy=(ntx>0)?XAxiDma_Busy(&AxiDma,XAXIDMA_DMA_TO_DEVICE):0;
    dmarxbusy=(nrx>0)?XAxiDma_Busy(&AxiDma,XAXIDMA_DEVICE_TO_DMA):0;
    printf("%04xB tx/rx block=%x/%x DMA tx/rx busy=%d/%d FIFO wr/rd cnt=%03x/%03x\r\n",
	   t,ntx,nrx,dmatxbusy,dmarxbusy,*GPIO_wc,*GPIO_rc);
    
    for (i=0;i<0x800000;i++)/* Delay */
      XGpioPs_WritePin(&gpiops,LED_PIN,((i&0x80000)?1:0)); /* Flash */
    
    t++;
  }

  /* ydvzcl`Ŏ󂯎̓LbVNA */
  Xil_DCacheInvalidateRange((u32)rxbuf, NUMBLK*BLKSIZE*sizeof(dataunit));

  r=XST_SUCCESS;
  for (i=0;i<NUMBLK;i++){
    if (i>=ntx||i>=nrx) break;
    for (j=0;j<BLKSIZE;j++) {
      if (rxbuf[i*BLKSIZE+j]!=txbuf[i*BLKSIZE+j]) {
	printf("data mismatch rxbuf[%02d][%02x]=%08x v.s. txbuf[%02x][%02x]=%08x\r\n",i,j,rxbuf[i*BLKSIZE+j], i,j,txbuf[i*BLKSIZE+j]);
	r=XST_FAILURE;
      }
    }
  }

  return r;
}
